package engine;

import java.util.*;
import java.io.*;
import engine.Plugin;
import java.util.regex.Pattern;
import java.util.regex.Matcher;
import functions.*;

class PluginLoader extends ClassLoader
{
	public Class findClass(String name) 
	{
		byte[] b = loadClassData(name);
		return defineClass(name, b, 0, b.length);
	}

	private byte[] loadClassData(String name) 
	{
		try
		{
			File inf = new File("plugins/" + name + ".class");
			FileInputStream ins = new FileInputStream(inf);
			byte buf[] = new byte[(int)inf.length()];
			ins.read(buf);
			ins.close();
			return buf;			
		}
		catch (Exception e)
		{
			System.err.println("Loader: plugins/" + name + ".class: " + e);
		}	
		return null;
	}
}

public class Manager
{
	private Vector<Plugin> mPlugins;
	private PluginLoader mLoader;
	private Vector<Vector<Object>> mRows;

	public Plugin lookupPlugin(String name)
	{
		for (Plugin plugin:mPlugins)
		{
			//System.out.println(name + " = " + plugin.getName());
			if (name.equals(plugin.getName()))
				return plugin;
		}
		return null;
	}

	public Manager()
	{
		mPlugins = new Vector<Plugin>();
		mLoader = new PluginLoader();
		mRows = new Vector<Vector<Object>>();

		/* Ladujemy wszystkie pluginy */
		File pluginsDir = new File("plugins");
		File list[] = pluginsDir.listFiles();
		for(int i=0;i<list.length;i++)
		{
			String name = list[i].getName();
			if (name.indexOf("Plugin.class") > 1)
				loadPlugin(name.replace("plugins.", "").replace(".class", ""));
		}
	}

	public boolean loadPlugin(String name)
	{
		try
		{
			Class plugin = mLoader.findClass("plugins." + name);
			Plugin instance = (Plugin)plugin.newInstance();
			mPlugins.add(instance);
			Boolean enabled = Settings.getValue(instance.getName() + "_ENABLED").equals("yes");
			if (enabled)
			{
				enablePlugin(instance.getName());
			}

			Vector<Object> row = new Vector<Object>();
			row.add(instance.getName());
			row.add((Object)enabled);
			mRows.add(row);
			
			return true;
		}
		catch (Exception e)
		{
			/* Blad */ 
			System.out.println("LoadPlugin: " + e);
			return false;
		}  
	}
 
	public boolean removePlugin(String name)
	{
		Plugin plugin = lookupPlugin(name);
		if (plugin == null)
			return false;
	
		mPlugins.remove(plugin);
		return true;
	}

	public boolean enablePlugin(String name)
	{
		Plugin plugin = lookupPlugin(name);
		if (plugin == null)
			return false;
		plugin.setEnabled(true);
		Settings.setValue(name + "_ENABLED", "yes");
		Settings.saveConfig();
		System.out.println("enablePlugin: " + name);
		return true;
	}

	public boolean disablePlugin(String name)
	{
		Plugin plugin = lookupPlugin(name);
		if (plugin == null)
			return false;
		plugin.setEnabled(false);
		Settings.setValue(name + "_ENABLED", "no");
		System.out.println("disablePlugin: " + name);
		return true;
	}
 
	public Plugin checkLink(String link)
	{
		for (Plugin plugin:mPlugins)
		{
			if ((plugin.enabled()) && (plugin.checkLink(link)))
				return plugin;   
		}
		return null;
	} 

	public Vector<Vector<Object>> getPlugins()
	{
		return mRows;
	}
};
