package plugins;

import engine.*;
import java.util.regex.*;
import java.net.*;
import java.io.*;

public class RapidsharePlugin implements Plugin
{
	private boolean mEnabled;
	private static int mActiveTasks = 0;

	public String getName()
	{
		return "Rapidshare.com";
	}

	public boolean enabled()	
	{
		return mEnabled;
	}

	public void setEnabled(boolean e)
	{
		mEnabled = e;
	}

	public boolean checkLink(String link)
	{
		return Pattern.matches("http://rapidshare.com/files/([0-9]+)/(.*)", link);
	}

	public boolean initTask(Task task)
	{	
		return true;
	}

	public void cleanTask(Task task)
	{
		
	}

	public String getIcon()
	{
		return "plugins/rs.png";
	}

	public boolean allowMoreTasks()
	{
		return (mActiveTasks < 1);
	}

	public void run(Task task)
	{
		URL url;
		URLConnection conn;
		InputStream input;
		byte buf[];
		String bufs;
		int secs = -1;
		String link;
		mActiveTasks++;
		
		/* Główna funkcja pluginu, tutaj po prostu pobieramy plik */
		try
		{
			/* Krok 1 -> pobieramy zawartośc strony z podanym linkiem */			
			conn = Task.makeConnection(task.getLink());
			if (conn == null)
			{
				task.setState(TaskState.tsError, "Błąd połączenia");
				mActiveTasks--;
				return;
			}
			
			url = conn.getURL();
			input = conn.getInputStream();
			Pattern p = Pattern.compile("(.*) action=\"(.*)\" method=(.*)");

			BufferedReader rd = new BufferedReader(new InputStreamReader(conn.getInputStream()));
			link = null;
			while ((bufs = rd.readLine()) != null) 
			{
				if (bufs.indexOf("The file could not be found") > 0)
				{
					task.setState(TaskState.tsError, "Plik nie istnieje");
					mActiveTasks--;
					return;
				}

				Matcher m = p.matcher(bufs);
				if (m.find())
				{
					link = m.group(2);
				}	
			}
			
			if (link == null)
			{
				task.setState(TaskState.tsError, "Dziwna odpowiedź serwera");
				mActiveTasks--;
				return;
			}			

			/* Krok 3 -> wysylamy formularz i pobieramy ile bedziemy czekac */
			conn = Task.makeConnection(link);			
			url = conn.getURL();
			String data = "dl.start=Free";
			conn.setDoOutput(true); 
			OutputStreamWriter wr = new OutputStreamWriter(conn.getOutputStream()); 
			wr.write(data); 
			wr.flush(); 
			
			rd = new BufferedReader(new InputStreamReader(conn.getInputStream()));
			Pattern p1 = Pattern.compile("(.*)var c=([0-9]+);(.*)");
			Pattern p2 = Pattern.compile("(.*)name=\"dlf\" action=\"(.*)\" method=\"post\"(.*)");
			link = null;
			while ((bufs = rd.readLine()) != null) 
			{
				Matcher m = p1.matcher(bufs);
				Matcher m2 = p2.matcher(bufs);
				if (m.find())
				{
					secs = Integer.parseInt(m.group(2)) + 1; /* +1 dla pewnosci ;) */
					continue;
				}

				if (m2.find())
				{
					link = m2.group(2);
				}

				if (bufs.indexOf("is already downloading a file") > 0)
				{
					task.setState(TaskState.tsWaiting, "Ściągasz inny plik");
					for (int i = 120; i >= 0; i--)
					{
						task.setETA(i);
						task.sleep(1000);
						if (task.getTaskState() != TaskState.tsWaiting)
						{
							mActiveTasks--;
							return;
						}
					}
					task.setState(TaskState.tsRunning);
					mActiveTasks--;
					return;
				}
			}

			if ((secs < 0) || (link == null))
			{
				task.setState(TaskState.tsError, "Dziwna odpowiedź serwera");
				mActiveTasks--;
				return;
			}
			
			/* Krok 4 -> Czekamy */
			task.setState(TaskState.tsWaiting);
			for(int i = secs; i >= 0; i--)
			{
 				task.setETA(i);
				task.sleep(1000);
				if (task.getTaskState() != TaskState.tsWaiting)
				{
					mActiveTasks--;
					return;
				}
			}
			task.setState(TaskState.tsRunning);

			/* Krok 5 -> Pobieramy właściwy plik \o/ */
			conn = Task.makeConnection(link);
			url = conn.getURL();
			data = "mirror=on";
			conn.setDoOutput(true); 
			wr = new OutputStreamWriter(conn.getOutputStream()); 
			wr.write(data); 
			wr.flush(); 
			input = conn.getInputStream();
			FileOutputStream out = new FileOutputStream(task.getDestDir() + File.separator + Task.baseName(url.getPath()));
			buf = new byte[8192];
			int len;
			int total = conn.getContentLength();
			int done = 0;

			while((len = input.read(buf)) > 0)
			{
				if (task.getTaskState() != TaskState.tsRunning)
				{	
					input.close();
					out.close();
					mActiveTasks--;
					return;
				}

				out.write(buf, 0, len);
				done += len;
				task.setProgress(done, total);				
			}

			input.close();
			out.close();
			task.setState(TaskState.tsDone);
		}
		catch (Exception e)
		{
			task.setState(TaskState.tsError);
		}
		mActiveTasks--;
	}
}
