/*main window class*/
package ui;

import javax.swing.*;
import javax.swing.table.*;
import java.awt.*;
import java.awt.event.*;
import ui.MenuBar;
import ui.ToolBar;
import java.util.*;
import functions.*;
import functions.ClipboardMonitor;
import ui.addlinks.AddLinks;
import functions.Settings;
import functions.ProgressBarRenderer;
import engine.*;

class TableUpdater extends Thread
{
	private JTable mTable;

	public TableUpdater(JTable t)
	{
		mTable = t;
	}

	public void run()
	{
		while(true)
		{
			mTable.updateUI();
			try
			{
				sleep(500);
			}
			catch (Exception e)
			{
			}
		}
	}
}

public class MainWindow extends JFrame
{
	@SuppressWarnings("unchecked") 

	private ui.MenuBar mMenu;
	private JPanel mPanel;
	private ui.ToolBar mTool;
	private JTable mTable;

	private AddLinks mDodaj;
	private static int i = 0;
	private ClipboardMonitor schowek;

	private Manager mPluginManager;
	private TableUpdater mTableUpdater;

	public MainWindow()
	{
		super("JGetRap - 0.1");

		Settings.loadConfig("data/config.jgrc");

		mPluginManager = new Manager();
		Task.setPluginManager(mPluginManager);

		createWindow();
		windowExit();
		addMenuBar();
		setIcon();
		addToolBar();  
		addTable();
		setBackground(Color.white);
		setVisible(true);
		

		if (Settings.getValue("NEW_VERSION_CHECK").equals("yes"))
		{
			Version v = new Version();
			Thread myThread = new Thread (v);
			myThread.start();
		}

		if (Settings.getValue("CLIPBOARD").equals("yes"))
		{
			schowek = new ClipboardMonitor(this);
			(new Thread(schowek)).start();
		}
		mTableUpdater = new TableUpdater(mTable);
		mTableUpdater.start();

		Task.loadLinks();
		Task.startNextTask();
	}
  
 
  private void setIcon(){
    ImageIcon ico = new ImageIcon("graphics/logo.png");
    setIconImage(ico.getImage());
  }

  private void createWindow(){
    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
    setSize(screenSize);
  }

	private void windowExit()
	{
		this.addWindowListener(new WindowAdapter()
		{
			public void windowClosing(WindowEvent e)
			{
				Task.saveLinks();
				System.exit(0);
			}
		});
	}

  private void addMenuBar(){
    mMenu = new ui.MenuBar(this);
    this.setJMenuBar(mMenu);
  }

  private void addToolBar(){
    mPanel = new JPanel();
    mTool = new ui.ToolBar(this);
    GridBagConstraints c = new GridBagConstraints();
    c.weightx = 1.0;
    c.weighty = 0.0;
    c.fill = GridBagConstraints.HORIZONTAL;
    c.gridy = 0;
    c.anchor = GridBagConstraints.PAGE_START;
    mPanel.setLayout(new GridBagLayout());
    mPanel.setSize(new Dimension(getWidth(), 0));
    mTool.setFloatable(false);
    mPanel.add(mTool,c);
    setContentPane(mPanel);
  }

	private void addTable()
	{
		Vector<String> titles = new Vector<String>();
		titles.add("");
		titles.add("Nazwa");
		titles.add("Postęp");
		titles.add("Status");
		titles.add("Priorytet");
		titles.add("Rozmiar");
		titles.add("Docelowy");
		titles.add("ETA");
  
		mTable= new JTable(new DefaultTableModel(Task.getTableRows(), titles)
		{
			@Override
			public boolean isCellEditable(int row, int column) {return false;}
		});

		mTable.setColumnSelectionAllowed(false);
		mTable.setCellSelectionEnabled(false);
		mTable.setRowSelectionAllowed(true);
		mTable.setShowGrid(false);
		mTable.setShowHorizontalLines(true) ;
		TableColumn col = mTable.getColumnModel().getColumn(0);
		col.setMaxWidth(16); 
		col.setMinWidth(16);
		mTable.getColumnModel().getColumn(0).setCellRenderer(new ImageRenderer(""));
		mTable.getColumnModel().getColumn(2).setCellRenderer(new ProgressBarRenderer("0"));
		GridBagConstraints c = new GridBagConstraints();
		c.weightx = 1.0;
		c.weighty = 10.0;
		c.anchor = GridBagConstraints.PAGE_START;
		c.fill = GridBagConstraints.BOTH;
		c.gridy = 1;
		JScrollPane scrollPane = new JScrollPane(mTable);
		mPanel.add(scrollPane, c);
	} 

	public MainWindow getMainWindow()
	{
		return this;
	}

	public ui.MenuBar getMenu()
	{
		return this.mMenu;
	}

	public JPanel getPanel()
	{
		return this.mPanel;
	}

	public ui.ToolBar getToolBar()
	{
		return this.mTool;
	}

	public JTable getTable()
	{
		return this.mTable;
	}

	public AddLinks getDodaj(){return this.mDodaj;}
	
	public void setDodaj(AddLinks dodaj){this.mDodaj=dodaj;}
	//0 zamkniete, >0 otwarte
	public int dodajIsOpen(){return this.i;}

	public void setDodajIsOpen(int j){i=j;}



	public void add(Vector<String> linki, String destiny, String Priority)
	{
		if(linki.size() > 0)
		{ 
			for(String str:linki)
			{
				Task.addTask(str, TaskState.tsStopped, TaskPriority.tpNormal, destiny);
			}
		}
	}


	public void remove()
	{
		int []idx = mTable.getSelectedRows();
		for(i=0;i<idx.length;i++)
		{
			Task.removeTask(idx[i]);
		}
	}

	public void pause()
	{
		int []idx = mTable.getSelectedRows();
		for(i=0;i<idx.length;i++)
		{
			Task.pauseTask(idx[i]);
		}
	}

	public void start()
	{
		int []idx = mTable.getSelectedRows();
		for(i=0;i<idx.length;i++)
		{
			Task.startTask(idx[i]);
		}		
	}
}
